/*
 * lilJudd
 * Copyright (C) 2023 moonleay
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

package net.moonleay.lilJudd.data.api.splatoon3ink

import net.moonleay.lilJudd.data.api.splatoon3ink.schedules.BankaraNode
import net.moonleay.lilJudd.data.api.splatoon3ink.schedules.RegularNode
import net.moonleay.lilJudd.data.api.splatoon3ink.schedules.XNode
import net.moonleay.lilJudd.util.TimeUtil

object Splatoon3ApiDataGrabber {
    private fun getRegularMode(timestamp: Long): RegularNode {
        Splatoon3Api.schedules!!.data.regularSchedules.nodes.map { modeData ->
            val startTime = TimeUtil.deformatJSONTime(modeData.startTime, "UTC")
            val endTime = TimeUtil.deformatJSONTime(modeData.endTime, "UTC")
            if (timestamp in startTime..endTime) {
                return modeData
            }
        }
        throw Exception("No current mode found")
    }

    private fun getOpenMode(timestamp: Long): BankaraNode {
        Splatoon3Api.schedules!!.data.bankaraSchedules.nodes.map { modeData ->
            val startTime = TimeUtil.deformatJSONTime(modeData.startTime, "UTC")
            val endTime = TimeUtil.deformatJSONTime(modeData.endTime, "UTC")
            if (timestamp in startTime..endTime) {
                modeData.bankaraMatchSettings!!.map { matchSetting ->
                    if (matchSetting.bankaraMode == "OPEN")
                        return modeData
                }
            }
        }
        throw Exception("No current mode found")
    }

    private fun getXMode(timestamp: Long): XNode {
        Splatoon3Api.schedules!!.data.xSchedules.nodes.map { modeData ->
            val startTime = TimeUtil.deformatJSONTime(modeData.startTime, "UTC")
            val endTime = TimeUtil.deformatJSONTime(modeData.endTime, "UTC")
            if (timestamp in startTime..endTime) {
                return modeData
            }
        }
        throw Exception("No current mode found")
    }

    private fun getSeriesMode(timestamp: Long): BankaraNode {
        Splatoon3Api.schedules!!.data.bankaraSchedules.nodes.map { modeData ->
            val startTime = TimeUtil.deformatJSONTime(modeData.startTime, "UTC")
            val endTime = TimeUtil.deformatJSONTime(modeData.endTime, "UTC")
            if (timestamp in startTime..endTime) {
                modeData.bankaraMatchSettings!!.map { matchSetting ->
                    if (matchSetting.bankaraMode == "CHALLENGE")
                        return modeData
                }
            }
        }
        throw Exception("No current mode found")
    }

    fun getRotationTime(timestamp: Long): String {
        val modeData = getRegularMode(timestamp)
        val endTime = TimeUtil.deformatJSONTime(modeData.endTime, "UTC")
        val diffStamp = TimeUtil.getTimeDifferenceFormatted(System.currentTimeMillis(), endTime)
        return "$diffStamp left in rotation"
    }

    fun getRegularMapsFormatted(timestamp: Long): String {
        val modeData = getRegularMode(timestamp)
        val map1 = modeData.regularMatchSetting.vsStages[0].name.split(" ")[0]
        val map2 = modeData.regularMatchSetting.vsStages[1].name.split(" ")[0]
        return "R: $map1, $map2"
    }

    fun getOpenMapFormatted(timestamp: Long): String {
        val modeData = getOpenMode(timestamp)
        // FIXME: This may cause issues if there is a Splatfest going on
        // TODO: Check if this works during splatfest
        // The Open Mode should always be the second mode in the list
        // FIXME: Add check if 2nd itm in list is open mode
        val map1 = modeData.bankaraMatchSettings!![1].vsStages[0].name.split(" ")[0]
        val map2 = modeData.bankaraMatchSettings[1].vsStages[1].name.split(" ")[0]
        return "O: ${modeData.bankaraMatchSettings[1].vsRule.name}: $map1, $map2"
            .replace("Rainmaker", "RMK")
            .replace("Tower Control", "TC")
            .replace("Splat Zones", "SZ")
            .replace("Clam Blitz", "CB")
    }

    fun getSeriesMapsFormatted(timestamp: Long): String {
        val modeData = getSeriesMode(timestamp)
        // FIXME: This may cause issues if there is a Splatfest going on
        // TODO: Check all the same things as in getOpenMapFormatted
        val map1 = modeData.bankaraMatchSettings!![0].vsStages[0].name.split(" ")[0]
        val map2 = modeData.bankaraMatchSettings[0].vsStages[1].name.split(" ")[0]
        return "S: ${modeData.bankaraMatchSettings[0].vsRule.name}: $map1, $map2"
            .replace("Rainmaker", "RMK")
            .replace("Tower Control", "TC")
            .replace("Splat Zones", "SZ")
            .replace("Clam Blitz", "CB")
    }

    fun getXMapFormatted(timestamp: Long): String {
        val modeData = getXMode(timestamp)
        val map1 = modeData.xMatchSetting.vsStages[0].name.split(" ")[0]
        val map2 = modeData.xMatchSetting.vsStages[1].name.split(" ")[0]
        return "X: ${modeData.xMatchSetting.vsRule.name}: $map1, $map2"
            .replace("Rainmaker", "RMK")
            .replace("Tower Control", "TC")
            .replace("Splat Zones", "SZ")
            .replace("Clam Blitz", "CB")
    }
}
